@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2019 Joris Vink <joris@coders.se>
+ * Copyright (c) 2013-2020 Joris Vink <joris@coders.se>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -30,11 +30,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#if !defined(KORE_NO_TLS)
 #include <openssl/err.h>
 #include <openssl/dh.h>
 #include <openssl/ssl.h>
-#endif
 
 #include <errno.h>
 #include <regex.h>
@@ -57,16 +55,31 @@ extern int daemon(int, int);
 #define st_mtim		st_mtimespec
 #endif
 
-#if !defined(KORE_NO_SENDFILE) && defined(KORE_NO_TLS)
+#if !defined(KORE_NO_SENDFILE)
 #if defined(__MACH__) || defined(__FreeBSD_version) || defined(__linux__)
 #define KORE_USE_PLATFORM_SENDFILE	1
 #endif
 #endif
 
+/*
+ * Figure out what type of OpenSSL API we are dealing with.
+ */
+#if defined(LIBRESSL_VERSION_NUMBER)
+#if LIBRESSL_VERSION_NUMBER >= 0x3000000fL
+#define KORE_OPENSSL_NEWER_API		1
+#endif
+#else
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+#define KORE_OPENSSL_NEWER_API		1
+#endif
+#endif
+
 #if defined(__OpenBSD__)
 #define KORE_USE_PLATFORM_PLEDGE	1
 #endif
 
+#define KORE_RSAKEY_BITS	4096
+
 #define KORE_RESULT_ERROR	0
 #define KORE_RESULT_OK		1
 #define KORE_RESULT_RETRY	2
@@ -75,6 +88,8 @@ extern int daemon(int, int);
 #define KORE_TLS_VERSION_1_2	1
 #define KORE_TLS_VERSION_BOTH	2
 
+#define KORE_BASE64_RAW		0x0001
+
 #define KORE_WAIT_INFINITE	(u_int64_t)-1
 #define KORE_RESEED_TIME	(1800 * 1000)
 
@@ -85,6 +100,10 @@ extern int daemon(int, int);
 #define KORE_PIDFILE_DEFAULT		"kore.pid"
 #define KORE_DEFAULT_CIPHER_LIST	"ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES256-GCM-SHA384:kEDH+AESGCM:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES256-SHA:DHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA:AES128-GCM-SHA256:AES256-GCM-SHA384:HIGH:!aNULL:!eNULL:!EXPORT:!DES:!3DES:!MD5:!PSK:!kRSA:!kDSA"
 
+#define KORE_CONFIG_HOOK	"kore_parent_configure"
+#define KORE_TEARDOWN_HOOK	"kore_parent_teardown"
+#define KORE_DAEMONIZED_HOOK	"kore_parent_daemonized"
+
 #if defined(KORE_DEBUG)
 #define kore_debug(...)		\
 	if (kore_debug)		\
@@ -107,15 +126,15 @@ extern int daemon(int, int);
 #define NETBUF_IS_STREAM	0x10
 #define NETBUF_IS_FILEREF	0x20
 
-#define X509_GET_CN(c, o, l)					\
-	X509_NAME_get_text_by_NID(X509_get_subject_name(c),	\
-	    NID_commonName, o, l)
+#define KORE_X509_COMMON_NAME_ONLY	0x0001
 
-#define X509_CN_LENGTH		(ub_common_name + 1)
+#define KORE_PEM_CERT_CHAIN	1
+#define KORE_DER_CERT_DATA	2
 
 /* XXX hackish. */
 #if !defined(KORE_NO_HTTP)
 struct http_request;
+struct http_redirect;
 #endif
 
 #define KORE_FILEREF_SOFT_REMOVED	0x1000
@@ -123,16 +142,14 @@ struct http_request;
 struct kore_fileref {
 	int				cnt;
 	int				flags;
+	int				ontls;
 	off_t				size;
 	char				*path;
 	u_int64_t			mtime;
 	time_t				mtime_sec;
 	u_int64_t			expiration;
-#if !defined(KORE_USE_PLATFORM_SENDFILE)
 	void				*base;
-#else
 	int				fd;
-#endif
 	TAILQ_ENTRY(kore_fileref)	list;
 };
 
@@ -145,12 +162,10 @@ struct netbuf {
 	u_int8_t		flags;
 
 	struct kore_fileref	*file_ref;
-#if defined(KORE_USE_PLATFORM_SENDFILE)
 	off_t			fd_off;
 	off_t			fd_len;
-#endif
 
-	void			*owner;
+	struct connection	*owner;
 	void			*extra;
 	int			(*cb)(struct netbuf *);
 
@@ -184,6 +199,8 @@ TAILQ_HEAD(netbuf_head, netbuf);
 #define CONN_CLOSE_EMPTY	0x02
 #define CONN_WS_CLOSE_SENT	0x04
 #define CONN_IS_BUSY		0x08
+#define CONN_ACME_CHALLENGE	0x10
+#define CONN_LOG_TLS_FAILURE	0x20
 
 #define KORE_IDLE_TIMER_MAX	5000
 
@@ -214,13 +231,12 @@ struct connection {
 	int			fd;
 	u_int8_t		state;
 	u_int8_t		proto;
-	void			*owner;
-#if !defined(KORE_NO_TLS)
+	struct listener		*owner;
 	X509			*cert;
 	SSL			*ssl;
+	char			*tls_sni;
 	int			tls_reneg;
-#endif
-	u_int8_t		flags;
+	u_int16_t		flags;
 	void			*hdlr_extra;
 
 	int			(*handle)(struct connection *);
@@ -284,18 +300,60 @@ struct kore_runtime_call {
 	struct kore_runtime	*runtime;
 };
 
+struct kore_domain {
+	u_int16_t				id;
+	int					logerr;
+	u_int64_t				logwarn;
+	int					accesslog;
+
+	char					*domain;
+	struct kore_buf				*logbuf;
+	struct kore_server			*server;
+
+#if defined(KORE_USE_ACME)
+	int					acme;
+	int					acme_challenge;
+	void					*acme_cert;
+	size_t					acme_cert_len;
+#endif
+	char					*cafile;
+	char					*crlfile;
+	char					*certfile;
+	char					*certkey;
+	SSL_CTX					*ssl_ctx;
+	int					x509_verify_depth;
+#if !defined(KORE_NO_HTTP)
+	TAILQ_HEAD(, kore_module_handle)	handlers;
+	TAILQ_HEAD(, http_redirect)		redirects;
+#endif
+	TAILQ_ENTRY(kore_domain)		list;
+};
+
+TAILQ_HEAD(kore_domain_h, kore_domain);
+
 extern struct kore_runtime	kore_native_runtime;
 
 struct listener {
 	struct kore_event		evt;
 	int				fd;
 	int				family;
+	char				*port;
+	char				*host;
+	struct kore_server		*server;
 	struct kore_runtime_call	*connect;
 
-	LIST_ENTRY(listener)	list;
+	LIST_ENTRY(listener)		list;
 };
 
-LIST_HEAD(listener_head, listener);
+struct kore_server {
+	int				tls;
+	char				*name;
+	struct kore_domain_h		domains;
+	LIST_HEAD(, listener)		listeners;
+	LIST_ENTRY(kore_server)		list;
+};
+
+LIST_HEAD(kore_server_list, kore_server);
 
 #if !defined(KORE_NO_HTTP)
 
@@ -350,7 +408,6 @@ struct kore_module {
 	char				*path;
 	char				*onload;
 	int				type;
-	time_t				mtime;
 	struct kore_runtime_call	*ocb;
 
 	struct kore_module_functions	*fun;
@@ -360,6 +417,7 @@ struct kore_module {
 };
 
 #if !defined(KORE_NO_HTTP)
+
 struct kore_module_handle {
 	char					*path;
 	char					*func;
@@ -389,8 +447,12 @@ struct kore_alog_header {
 } __attribute__((packed));
 
 struct kore_worker {
-	u_int8_t			id;
-	u_int8_t			cpu;
+	u_int16_t			id;
+	u_int16_t			cpu;
+	int				running;
+#if defined(__linux__)
+	int				tracing;
+#endif
 	pid_t				pid;
 	int				pipe[2];
 	struct connection		*msg[2];
@@ -407,29 +469,6 @@ struct kore_worker {
 	} lb;
 };
 
-struct kore_domain {
-	u_int16_t				id;
-	char					*domain;
-	int					accesslog;
-	struct kore_buf				*logbuf;
-	int					logerr;
-	u_int64_t				logwarn;
-#if !defined(KORE_NO_TLS)
-	char					*cafile;
-	char					*crlfile;
-	char					*certfile;
-	char					*certkey;
-	SSL_CTX					*ssl_ctx;
-	int					x509_verify_depth;
-#endif
-#if !defined(KORE_NO_HTTP)
-	TAILQ_HEAD(, kore_module_handle)	handlers;
-#endif
-	TAILQ_ENTRY(kore_domain)		list;
-};
-
-TAILQ_HEAD(kore_domain_h, kore_domain);
-
 #if !defined(KORE_NO_HTTP)
 
 #define KORE_VALIDATOR_TYPE_REGEX	1
@@ -455,6 +494,91 @@ struct kore_buf {
 	size_t			offset;
 };
 
+#define KORE_JSON_TYPE_OBJECT		1
+#define KORE_JSON_TYPE_ARRAY		2
+#define KORE_JSON_TYPE_STRING		3
+#define KORE_JSON_TYPE_NUMBER		4
+#define KORE_JSON_TYPE_LITERAL		5
+
+#define KORE_JSON_FALSE			0
+#define KORE_JSON_TRUE			1
+#define KORE_JSON_NULL			2
+
+#define KORE_JSON_DEPTH_MAX		10
+
+#define KORE_JSON_ERR_NONE		0
+#define KORE_JSON_ERR_INVALID_OBJECT	1
+#define KORE_JSON_ERR_INVALID_ARRAY	2
+#define KORE_JSON_ERR_INVALID_STRING	3
+#define KORE_JSON_ERR_INVALID_NUMBER	4
+#define KORE_JSON_ERR_INVALID_LITERAL	5
+#define KORE_JSON_ERR_DEPTH		6
+#define KORE_JSON_ERR_EOF		7
+#define KORE_JSON_ERR_INVALID_JSON	8
+#define KORE_JSON_ERR_INVALID_SEARCH	9
+#define KORE_JSON_ERR_NOT_FOUND		10
+#define KORE_JSON_ERR_TYPE_MISMATCH	11
+#define KORE_JSON_ERR_LAST		KORE_JSON_ERR_TYPE_MISMATCH
+
+#define kore_json_find_object(j, p)		\
+    kore_json_find(j, p, KORE_JSON_TYPE_OBJECT)
+
+#define kore_json_find_array(j, p)		\
+    kore_json_find(j, p, KORE_JSON_TYPE_ARRAY)
+
+#define kore_json_find_string(j, p)		\
+    kore_json_find(j, p, KORE_JSON_TYPE_STRING)
+
+#define kore_json_find_number(j, p)		\
+    kore_json_find(j, p, KORE_JSON_TYPE_NUMBER)
+
+#define kore_json_find_literal(j, p)		\
+    kore_json_find(j, p, KORE_JSON_TYPE_LITERAL)
+
+#define kore_json_create_object(o, n)				\
+    kore_json_create_item(o, n, KORE_JSON_TYPE_OBJECT)
+
+#define kore_json_create_array(o, n)				\
+    kore_json_create_item(o, n, KORE_JSON_TYPE_ARRAY)
+
+#define kore_json_create_string(o, n, v)			\
+    kore_json_create_item(o, n, KORE_JSON_TYPE_STRING, v)
+
+#define kore_json_create_number(o, n, v)			\
+    kore_json_create_item(o, n, KORE_JSON_TYPE_NUMBER, v)
+
+#define kore_json_create_literal(o, n, v)			\
+    kore_json_create_item(o, n, KORE_JSON_TYPE_LITERAL, v)
+
+struct kore_json {
+	const u_int8_t			*data;
+	int				depth;
+	int				error;
+	size_t				length;
+	size_t				offset;
+
+	struct kore_buf			tmpbuf;
+	struct kore_json_item		*root;
+};
+
+struct kore_json_item {
+	int				type;
+	char				*name;
+	struct kore_json_item		*parent;
+
+	union {
+		TAILQ_HEAD(, kore_json_item)	items;
+		char				*string;
+		double				number;
+		int				literal;
+	} data;
+
+	int				(*parse)(struct kore_json *,
+					    struct kore_json_item *);
+
+	TAILQ_ENTRY(kore_json_item)	list;
+};
+
 struct kore_pool_region {
 	void				*start;
 	size_t				length;
@@ -490,7 +614,17 @@ struct kore_timer {
 	TAILQ_ENTRY(kore_timer)	list;
 };
 
-#define KORE_WORKER_KEYMGR		0
+/*
+ * Keymgr process is worker index 0, but id 2000.
+ * Acme process is worker index 1, but id 2001.
+ */
+#define KORE_WORKER_KEYMGR_IDX		0
+#define KORE_WORKER_ACME_IDX		1
+#define KORE_WORKER_BASE		2
+#define KORE_WORKER_KEYMGR		2000
+#define KORE_WORKER_ACME		2001
+#define KORE_WORKER_MAX			UCHAR_MAX
+
 #define KORE_WORKER_POLICY_RESTART	1
 #define KORE_WORKER_POLICY_TERMINATE	2
 
@@ -505,6 +639,11 @@ struct kore_timer {
 #define KORE_MSG_CERTIFICATE_REQ	8
 #define KORE_MSG_CRL			9
 #define KORE_MSG_ACCEPT_AVAILABLE	10
+#define KORE_PYTHON_SEND_OBJ		11
+#define KORE_MSG_ACME_BASE		100
+
+/* messages for applications should start at 201. */
+#define KORE_MSG_APP_BASE		200
 
 /* Predefined message targets. */
 #define KORE_MSG_PARENT		1000
@@ -517,22 +656,18 @@ struct kore_msg {
 	size_t		length;
 };
 
-#if !defined(KORE_NO_TLS)
 struct kore_keyreq {
 	int		padding;
-	char		domain[KORE_DOMAINNAME_LEN];
-	u_int16_t	domain_len;
-	u_int16_t	data_len;
+	char		domain[KORE_DOMAINNAME_LEN + 1];
+	size_t		data_len;
 	u_int8_t	data[];
 };
 
 struct kore_x509_msg {
-	char		domain[KORE_DOMAINNAME_LEN];
-	u_int16_t	domain_len;
+	char		domain[KORE_DOMAINNAME_LEN + 1];
 	size_t		data_len;
 	u_int8_t	data[];
 };
-#endif
 
 #if !defined(KORE_SINGLE_BINARY)
 extern char	*config_file;
@@ -551,13 +686,14 @@ extern char	*kore_tls_cipher_list;
 
 extern volatile sig_atomic_t	sig_recv;
 
-#if !defined(KORE_NO_TLS)
 extern int	tls_version;
 extern DH	*tls_dhparam;
 extern char	*rand_file;
+extern int	keymgr_active;
 extern char	*keymgr_runas_user;
 extern char	*keymgr_root_path;
-#endif
+extern char	*acme_runas_user;
+extern char	*acme_root_path;
 
 extern u_int8_t			nlisteners;
 extern u_int16_t		cpu_count;
@@ -573,11 +709,10 @@ extern u_int64_t		kore_websocket_maxframe;
 extern u_int64_t		kore_websocket_timeout;
 extern u_int32_t		kore_socket_backlog;
 
-extern struct listener_head	listeners;
 extern struct kore_worker	*worker;
-extern struct kore_domain_h	domains;
-extern struct kore_domain	*primary_dom;
 extern struct kore_pool		nb_pool;
+extern struct kore_domain	*primary_dom;
+extern struct kore_server_list	kore_servers;
 
 void		kore_signal(int);
 void		kore_shutdown(void);
@@ -589,22 +724,28 @@ void		kore_worker_init(void);
 void		kore_worker_make_busy(void);
 void		kore_worker_shutdown(void);
 void		kore_worker_dispatch_signal(int);
-void		kore_worker_spawn(u_int16_t, u_int16_t);
-void		kore_worker_entry(struct kore_worker *);
 void		kore_worker_privdrop(const char *, const char *);
+void		kore_worker_spawn(u_int16_t, u_int16_t, u_int16_t);
+int		kore_worker_keymgr_response_verify(struct kore_msg *,
+		    const void *, struct kore_domain **);
+
+void	kore_worker_entry(struct kore_worker *) __attribute__((noreturn));
 
 struct kore_worker	*kore_worker_data(u_int8_t);
 
 void		kore_platform_init(void);
+void		kore_platform_sandbox(void);
 void		kore_platform_event_init(void);
 void		kore_platform_event_cleanup(void);
-void		kore_platform_proctitle(char *);
 void		kore_platform_disable_read(int);
 void		kore_platform_disable_write(int);
 void		kore_platform_enable_accept(void);
 void		kore_platform_disable_accept(void);
 void		kore_platform_event_wait(u_int64_t);
 void		kore_platform_event_all(int, void *);
+void		kore_platform_event_level_all(int, void *);
+void		kore_platform_event_level_read(int, void *);
+void		kore_platform_proctitle(const char *);
 void		kore_platform_schedule_read(int, void *);
 void		kore_platform_schedule_write(int, void *);
 void		kore_platform_event_schedule(int, int, int, void *);
@@ -641,18 +782,28 @@ void		kore_timer_remove(struct kore_timer *);
 struct kore_timer	*kore_timer_add(void (*cb)(void *, u_int64_t),
 			    u_int64_t, void *, int);
 
-void		kore_listener_cleanup(void);
+void		kore_server_closeall(void);
+void		kore_server_cleanup(void);
+void		kore_server_free(struct kore_server *);
+void		kore_server_finalize(struct kore_server *);
+
+struct kore_server	*kore_server_create(const char *);
+struct kore_server	*kore_server_lookup(const char *);
+
 void		kore_listener_accept(void *, int);
+struct listener	*kore_listener_lookup(const char *);
 void		kore_listener_free(struct listener *);
-struct listener	*kore_listener_alloc(int, const char *);
+struct listener	*kore_listener_create(struct kore_server *);
+int		kore_listener_init(struct listener *, int, const char *);
 
 int		kore_sockopt(int, int, int);
-int		kore_server_bind_unix(const char *, const char *);
-int		kore_server_bind(const char *, const char *, const char *);
-#if !defined(KORE_NO_TLS)
+int		kore_server_bind_unix(struct kore_server *,
+		    const char *, const char *);
+int		kore_server_bind(struct kore_server *,
+		    const char *, const char *, const char *);
+
 int		kore_tls_sni_cb(SSL *, int *, void *);
 void		kore_tls_info_callback(const SSL *, int, int);
-#endif
 
 void			kore_connection_init(void);
 void			kore_connection_cleanup(void);
@@ -674,6 +825,10 @@ int			kore_connection_accept(struct listener *,
 u_int64_t	kore_time_ms(void);
 void		kore_log_init(void);
 
+#if defined(KORE_USE_PYTHON)
+int		kore_configure_setting(const char *, char *);
+#endif
+
 void		*kore_malloc(size_t);
 void		kore_parse_config(void);
 void		kore_parse_config_file(FILE *);
@@ -708,10 +863,16 @@ long long	kore_strtonum(const char *, int, long long, long long, int *);
 double		kore_strtodouble(const char *, long double, long double, int *);
 int		kore_base64_encode(const void *, size_t, char **);
 int		kore_base64_decode(const char *, u_int8_t **, size_t *);
+int		kore_base64url_encode(const void *, size_t, char **, int);
+int		kore_base64url_decode(const char *, u_int8_t **, size_t *, int);
 void		*kore_mem_find(void *, size_t, const void *, size_t);
 char		*kore_text_trim(char *, size_t);
 char		*kore_read_line(FILE *, char *, size_t);
 
+EVP_PKEY	*kore_rsakey_load(const char *);
+EVP_PKEY	*kore_rsakey_generate(const char *);
+int		kore_x509_subject_name(struct connection *, char **, int);
+
 #if !defined(KORE_NO_HTTP)
 void		kore_websocket_handshake(struct http_request *,
 		    const char *, const char *, const char *);
@@ -725,6 +886,7 @@ void		kore_websocket_broadcast(struct connection *,
 void		kore_msg_init(void);
 void		kore_msg_worker_init(void);
 void		kore_msg_parent_init(void);
+void		kore_msg_unregister(u_int8_t);
 void		kore_msg_parent_add(struct kore_worker *);
 void		kore_msg_parent_remove(struct kore_worker *);
 void		kore_msg_send(u_int16_t, u_int8_t, const void *, size_t);
@@ -741,14 +903,15 @@ extern char	*kore_filemap_index;
 #endif
 
 void			kore_fileref_init(void);
-struct kore_fileref	*kore_fileref_get(const char *);
-struct kore_fileref	*kore_fileref_create(const char *, int, off_t,
-			    struct timespec *);
+struct kore_fileref	*kore_fileref_get(const char *, int);
+struct kore_fileref	*kore_fileref_create(struct kore_server *,
+			    const char *, int, off_t, struct timespec *);
 void			kore_fileref_release(struct kore_fileref *);
 
+struct kore_domain	*kore_domain_new(const char *);
+
 void		kore_domain_init(void);
 void		kore_domain_cleanup(void);
-int		kore_domain_new(char *);
 void		kore_domain_free(struct kore_domain *);
 void		kore_module_init(void);
 void		kore_module_cleanup(void);
@@ -760,14 +923,16 @@ void		*kore_module_getsym(const char *, struct kore_runtime **);
 void		kore_domain_load_crl(void);
 void		kore_domain_keymgr_init(void);
 void		kore_domain_callback(void (*cb)(struct kore_domain *));
-void		kore_domain_tlsinit(struct kore_domain *, const void *, size_t);
+int		kore_domain_attach(struct kore_domain *, struct kore_server *);
+void		kore_domain_tlsinit(struct kore_domain *, int,
+		    const void *, size_t);
 void		kore_domain_crl_add(struct kore_domain *, const void *, size_t);
 #if !defined(KORE_NO_HTTP)
-int		kore_module_handler_new(const char *, const char *,
+int		kore_module_handler_new(struct kore_domain *, const char *,
 		    const char *, const char *, int);
 void		kore_module_handler_free(struct kore_module_handle *);
-struct kore_module_handle	*kore_module_handler_find(const char *,
-				    const char *);
+struct kore_module_handle	*kore_module_handler_find(struct http_request *,
+				    struct kore_domain *);
 #endif
 
 struct kore_runtime_call	*kore_runtime_getcall(const char *);
@@ -790,8 +955,8 @@ void	kore_runtime_wsmessage(struct kore_runtime_call *,
 	    struct connection *, u_int8_t, const void *, size_t);
 #endif
 
-struct kore_domain		*kore_domain_byid(u_int16_t);
-struct kore_domain		*kore_domain_lookup(const char *);
+struct kore_domain	*kore_domain_byid(u_int16_t);
+struct kore_domain	*kore_domain_lookup(struct kore_server *, const char *);
 
 #if !defined(KORE_NO_HTTP)
 void		kore_validator_init(void);
@@ -805,6 +970,8 @@ struct kore_validator	*kore_validator_lookup(const char *);
 
 void		fatal(const char *, ...) __attribute__((noreturn));
 void		fatalx(const char *, ...) __attribute__((noreturn));
+
+const char	*kore_worker_name(int);
 void		kore_debug_internal(char *, int, const char *, ...);
 
 u_int16_t	net_read16(u_int8_t *);
@@ -850,9 +1017,22 @@ void	kore_buf_appendv(struct kore_buf *, const char *, va_list);
 void	kore_buf_replace_string(struct kore_buf *,
 	    const char *, const void *, size_t);
 
+int	kore_json_parse(struct kore_json *);
+void	kore_json_cleanup(struct kore_json *);
+void	kore_json_item_free(struct kore_json_item *);
+void	kore_json_init(struct kore_json *, const u_int8_t *, size_t);
+void	kore_json_item_tobuf(struct kore_json_item *, struct kore_buf *);
+
+const char		*kore_json_strerror(struct kore_json *);
+struct kore_json_item	*kore_json_find(struct kore_json_item *,
+			    const char *, int);
+struct kore_json_item	*kore_json_create_item(struct kore_json_item *,
+			    const char *, int, ...);
+
 void	kore_keymgr_run(void);
 void	kore_keymgr_cleanup(int);
 
+void	kore_seccomp_hook(void);
 void	kore_worker_teardown(void);
 void	kore_parent_teardown(void);
 void	kore_worker_configure(void);
